@@ -2,12 +2,12 @@
 use strict;
 $| = 1;
 
-# $Id: archiverpt.pl 858 2005-05-01 22:19:51Z abeltje $
+# $Id: archiverpt.pl 637 2004-03-03 17:52:55Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.004';
+$VERSION = '0.003';
 
 use Cwd;
-use File::Spec::Functions;
+use File::Spec;
 use File::Path;
 use File::Copy;
 use FindBin;
@@ -118,16 +118,14 @@ $opt{adir} && -d $opt{adir} or do {
 my $patch_level = get_patch( $opt{ddir} );
 $patch_level =~ tr/ //sd;
 
-my $archived_rpt = catfile( $opt{adir}, "rpt${patch_level}.rpt" );
+my $archived_rpt = "rpt${patch_level}.rpt";
 
 # Do not archive if it is already done
--f $archived_rpt && !$opt{force} and exit;
+-f File::Spec->catfile( $opt{adir}, $archived_rpt ) && $opt{force} and exit;
 
-my $mktest_rpt = catfile( $opt{ddir}, 'mktest.rpt' );
-if ( -f $mktest_rpt ) {
-    copy( $mktest_rpt, $archived_rpt ) or
-        die "Cannot copy to '$archived_rpt': $!";
-}
+copy( File::Spec->catfile( $opt{ddir}, 'mktest.rpt' ), 
+      File::Spec->catfile( $opt{adir}, $archived_rpt ) ) or
+    die "Cannot copy to '$archived_rpt': $!";
 
 SKIP_LOG: {
     my $archived_log = "log${patch_level}.log";
