@@ -5,15 +5,16 @@ use Config;
 use Cwd;
 use File::Spec;
 use File::Path;
+use File::Copy;
 use Data::Dumper;
 use FindBin;
 use lib File::Spec->catdir( $FindBin::Bin, 'lib' );
 use lib $FindBin::Bin;
 use Test::Smoke::Util qw( do_pod2usage );
 
-# $Id: configsmoke.pl 319 2003-08-03 11:24:26Z abeltje $
+# $Id: configsmoke.pl 705 2004-05-04 20:08:07Z abeltje $
 use vars qw( $VERSION $conf );
-$VERSION = '0.025';
+$VERSION = '0.042';
 
 use Getopt::Long;
 my %options = ( 
@@ -23,11 +24,12 @@ my %options = (
     default => undef,
     prefix  => undef,
     oldcfg  => 0,
+    usedft  => undef,
 );
 my $myusage = "Usage: $0 -p <prefix>[ -d <defaultsprefix>]";
 GetOptions( \%options, 
     'config|c=s', 'jcl|j=s', 'log|l=s', 
-    'prefix|p=s', 'default|d=s',
+    'prefix|p=s', 'default|d:s', 'usedft|des',
 
     'help|h', 'man',
 ) or do_pod2usage( verbose => 1, myusage => $myusage );
@@ -57,13 +59,27 @@ if ( $@ || $options{default} ) {
         unless $@;
 } 
 
+# -des will only work fully when $options{oldcfg}
+unless ( $options{oldcfg} ) {
+    defined $options{usedft}
+        and print "Option '-des' not fully functional!\n";
+} else {
+    # -d like in ./Configure, works for oldcfg only!
+    $options{usedft} ||= defined $options{default} &&
+                         $options{default} eq "";
+}
+
 =head1 NAME
 
 configsmoke.pl - Create a configuration for B<smokeperl.pl>
 
 =head1 SYNOPSIS
 
-   $ perl configsmoke.pl -p <prefix>[ -d <defaultsprefix>]
+    $ perl configsmoke.pl -p <prefix>[ -d <defaultsprefix>]
+
+or regenerate from previous _config:
+
+    $ perl configsmoke.pl -p <prefix> -des 
 
 =head1 OPTIONS
 
@@ -75,6 +91,8 @@ Current options:
   -l logprefix      When ommited 'perlcurrent' is used
   -p prefix         Set -c and -j and -l at once
 
+  -des              confirm all answers (needs previous _config)
+
 =cut
 
 sub is_win32() { $^O eq 'MSWin32' }
@@ -94,7 +112,22 @@ my @untars = get_avail_tar();
 my $untarmsg = join "", map "\n\t$_" => @untars;
 
 my %versions = (
+    '5.5.x' => { source => 'ftp.linux.activestate.com::perl-5.005xx',
+                 server => 'ftp.funet.fi',
+                 sdir   => '/pub/languages/perl/snap/5.5.x',
+                 sfile  => '',
+                 pdir   => '/pub/staff/gsar/APC/perl-5.005xx-diffs',
+                 cfg    => 'perl55x.cfg',
+                 ddir   => File::Spec->rel2abs( 
+                               File::Spec->catdir( File::Spec->updir,
+                                                   'perl-5.5.x' ) ),
+                 text   => 'Perl 5.005 MAINT',
+                 is56x  => 1},
     '5.6.2' => { source => 'ftp.linux.activestate.com::perl-5.6.2',
+                 server => 'http://rgarciasuarez.free.fr',
+                 sdir   => '/snap',
+                 sfile  => 'perl562-21463.tar.gz',
+                 pdir   => '/pub/staff/gsar/APC/perl-5.6.2-diffs',
                  ddir   => File::Spec->rel2abs( 
                                File::Spec->catdir( File::Spec->updir,
                                                    'perl-5.6.2' ) ),
@@ -102,26 +135,26 @@ my %versions = (
                  text   => 'Perl 5.6.2-to-be',
                  is56x  => 1 },
     '5.8.x' => { source =>  'ftp.linux.activestate.com::perl-5.8.x',
-                 server => 'http://www.iki.fi',
-                 sdir   => '/jhi',
-                 sfile  => 'perl@19856.tgz',
+                 server => 'ftp.funet.fi',
+                 sdir   => '/pub/languages/perl/snap/5.8.x',
+                 sfile  => '',
                  pdir   => '/pub/staff/gsar/APC/perl-5.8.x-diffs',
                  ddir   => File::Spec->rel2abs( 
                                File::Spec->catdir( File::Spec->updir,
                                                    'perl-5.8.x' ) ),
-                 text   => 'Perl 5.8.1-to-be',
+                 text   => 'Perl 5.8 MAINT',
                  cfg    => ( $^O eq 'MSWin32' 
-                        ? 'w32current.cfg' :'perlcurrent.cfg' ),
+                        ? 'w32current.cfg' :'perl58x.cfg' ),
                  is56x  => 0 },
     '5.9.x' => { source => 'ftp.linux.activestate.com::perl-current',
                  server => 'ftp.funet.fi',
-                 sdir   => '/pub/languages/perl/snap/',
+                 sdir   => '/pub/languages/perl/snap/5.9.x',
                  sfile  => '',
                  pdir   => '/pub/staff/gsar/APC/perl-current-diffs',
                  ddir   => File::Spec->rel2abs( 
                                File::Spec->catdir( File::Spec->updir,
                                                    'perl-current' ) ),
-                 text   => 'Perl 5.10.0-to-be',
+                 text   => 'Perl 5.10 to-be',
                  cfg    => ( $^O eq 'MSWin32' 
                         ? 'w32current.cfg' :'perlcurrent.cfg' ),
                  is56x  => 0 },
@@ -152,6 +185,7 @@ my %opt = (
         alt => [ ],
         dft => File::Spec->rel2abs( File::Spec->catdir( File::Spec->updir,
                                                         'perl-current' ) ),
+        chk => '.+',
     },
     use_old => {
         msg => "It looks like there is already a source-tree there.\n" .
@@ -199,12 +233,14 @@ my %opt = (
         alt => [ ],
         dft => File::Spec->rel2abs( File::Spec->catdir( File::Spec->updir,
                                                         'perl-master' ) ),
+        chk => '.+',
     },
     forest_hdir => {
         msg => 'Where would you like the intermediate source-tree?',
         alt => [ ],
         dft => File::Spec->rel2abs( File::Spec->catdir( File::Spec->updir,
                                                         'perl-inter' ) ),
+        chk => '.+',
     },
     fsync => { 
         msg => "How would you like to sync your master source-tree?\n$syncmsg",
@@ -300,18 +336,20 @@ Examples:$untarmsg",
         msg => 'From which directory should the source-tree be copied?',
         alt => [ ],
         dft => undef,
+        chk => '.+',
     },
 
     hdir => {
         msg => 'From which directory should the source-tree be hardlinked?',
         alt => [ ],
         dft => undef,
+        chk => '.+',
     },
 
-    patch => {
+    patchbin => {
         msg => undef,
         alt => [ ],
-        dft => whereis( 'gpatch') || whereis( 'patch' ),
+        dft => find_a_patch(),
     },
 
     popts => {
@@ -328,11 +366,28 @@ Examples:$untarmsg",
         dft => ''
     },
 
+    # make fine-tuning
+    makeopt => {
+        msg => <<EOT,
+Specify extra arguments for "$Config{make}".
+\t(for the build step and test-prep step)
+EOT
+        alt => [ ],
+        dft => '',
+    },
+    testmake => {
+        msg => <<EOT,
+Specify a different make program for "make _test".
+EOT
+        alt => [ ],
+        dft => $Config{make},
+    },
+
     # mail stuff
     mail => {
         msg => "Would you like your reports send by e-mail?",
-        alt => [qw( Y n )],
-        dft => 'y',
+        alt => [qw( N y )],
+        dft => 'n',
     },
     mail_type => {
         msg => 'Which mail facility should be used?',
@@ -348,17 +403,42 @@ Examples:$untarmsg",
     },
 
     to => {
-       msg => "To which address(es) should the report be send " .
-              "(comma separated list)?",
+       msg => <<EOMSG,
+To which address(es) should the report *always* be send?
+\t(comma separated list, *please* do not include perl5-porters!)
+EOMSG
        alt => [ ],
        dft => 'smokers-reports@perl.org',
+       nck => '\bperl5-porters@perl.org\b',
+    },
+
+    bcc => {
+       msg => <<EOMSG,
+To which address(es) should the report *always* be BCCed?
+\t(comma separated list, *please* do not include perl5-porters!)
+EOMSG
+       alt => [ ],
+       dft => '',
+       nck => '\bperl5-porters@perl.org\b',
     },
 
     cc => {
-       msg => "To which address(es) should the report be CC'ed " .
-              "(comma separated list)?",
+       msg => <<EOMSG,
+* THIS FEATURE HAS CHANGED IN 1.19! *
+To which address(es) should the report be CCed *on fail*?
+\t(comma separated list, *please* do not include perl5-porters!)
+EOMSG
        alt => [ ],
        dft => '',
+       nck => '\bperl5-porters@perl.org\b',
+    },
+
+    ccp5p_onfail => {
+        msg => <<EOMSG,
+Would you like your failed smoke reports CCed to perl5-porters?
+EOMSG
+        alt => [qw( y N )],
+        dft => 'n',
     },
 
     from => {
@@ -389,19 +469,30 @@ Examples:$untarmsg",
         dft => 'y',
     },
     killtime => {
-        msg => "Should this smoke be aborted on/after a specific time?
+        msg => <<EOT,
+Should this smoke be aborted on/after a specific time?
 \tuse HH:MM to specify a point in time (24 hour notation)
 \tuse +HH:MM to specify a duration
-\tleave empty to finish the smoke without aborting",
+\tleave empty to finish the smoke without aborting
+EOT
         dft => "",
         alt => [ ],
         chk => '^(?:(?:\+\d+)|(?:(?:[0-1]?[0-9])|(?:2[0-3])):[0-5]?[0-9])|$',
     },
+    # Archive?
+    adir => {
+        msg => <<EOT,
+Which directory should be used for the archives?
+\tLeave empty for no archiving.
+EOT
+        alt => [ ],
+        dft => "",
+    },
     # Schedule stuff
     docron => {
         msg => 'Should the smoke be scheduled?',
         alt => [qw( Y n )],
-        dft => 'y',
+        dft => 'n',
     },
     crontime => {
         msg => 'At what time should the smoke be scheduled?',
@@ -459,7 +550,7 @@ There are two additional configuration default files
 F<smoke562_dfconfig> and F<smoke58x_dfconfig> to help you configure 
 B<Test::Smoke> for these two maintenance branches of the source-tree.
 
-To create a configuration for the perl 5.8.x brach:
+To create a configuration for the perl 5.8.x branch:
 
     $ perl configsmoke.pl -p smoke58x
 
@@ -533,30 +624,35 @@ you will need to confirm your choice.
 
 =cut
 
-BUILDDIR: {
-    $arg = 'ddir';
-    $config{ $arg } = prompt_dir( $arg );
-    my $cwd = cwd;
-    unless ( chdir $config{ $arg } ) {
-        warn "Can't chdir($config{ $arg }): $!\n";
-        redo BUILDDIR;
-    }
-    my $bdir = $config{ $arg } = cwd;
-    chdir $cwd or die "Can't chdir($cwd) back: $!\n";
-    if ( $cwd eq $bdir ) {
-        print "The current directory *cannot* be used for smoke testing\n";
-        redo BUILDDIR;
-    }
-
-    $config{ $arg } = File::Spec->canonpath( $config{ $arg } );
-    my $manifest  = File::Spec->catfile( $config{ $arg }, 'MANIFEST' );
-    my $dot_patch = File::Spec->catfile( $config{ $arg }, '.patch' );
-    if ( -e $manifest && -e $dot_patch ) {
-        $opt{use_old}->{dft} = $options{oldcfg} && 
-                               ($conf->{ddir}||"") eq $config{ddir}
-            ? 'y' : $opt{use_old}->{dft};
-        my $use_old = prompt_yn( 'use_old' );
-        redo BUILDDIR unless $use_old;
+{
+    # Hack -des to keep the safeguards
+    local $options{usedft} = $options{usedft};
+    BUILDDIR: {
+        $arg = 'ddir';
+        $config{ $arg } = prompt_dir( $arg );
+        $options{usedft} = $options{oldcfg};
+        my $cwd = cwd;
+        unless ( chdir $config{ $arg } ) {
+            warn "Can't chdir($config{ $arg }): $!\n";
+            redo BUILDDIR;
+        }
+        my $bdir = $config{ $arg } = cwd;
+        chdir $cwd or die "Can't chdir($cwd) back: $!\n";
+        if ( $cwd eq $bdir ) {
+            print "The current directory *cannot* be used for smoke testing\n";
+            redo BUILDDIR;
+        }
+    
+        $config{ $arg } = File::Spec->canonpath( $config{ $arg } );
+        my $manifest  = File::Spec->catfile( $config{ $arg }, 'MANIFEST' );
+        my $dot_patch = File::Spec->catfile( $config{ $arg }, '.patch' );
+        if ( -e $manifest && -e $dot_patch ) {
+            $opt{use_old}->{dft} = $options{oldcfg} && 
+                                   ($conf->{ddir}||"") eq $config{ddir}
+                ? 'y' : $opt{use_old}->{dft};
+            my $use_old = prompt_yn( 'use_old' );
+            redo BUILDDIR unless $use_old;
+        }
     }
 }
 
@@ -567,12 +663,16 @@ There are several build-cfg files provided with the distribution:
 
 =over 4
 
-=item F<perlcurrent.cfg> for 5.8.x+ on unixy systems
+=item F<perlcurrent.cfg> for 5.9.x+ on unixy systems
 
-=item F<w32current.cfg> for 5.8.x+ on MSWin32
+=item F<perl58x.cfg> for 5.8.x (MAINT) on unixy systems
 
 =item F<perl562.cfg> for 5.6.2 (MAINT) on unixy systems
 
+=item F<perl55x.cfg> for 5.005_04 (MAINT) on unixy systems
+
+=item F<w32current.cfg> for 5.8.x+ on MSWin32
+
 =back
 
 Note: 5.6.2 on MSWin32 is not yet provided, but commenting out the
@@ -581,6 +681,7 @@ B<-Duselargefiles> section from F<w32current.cfg> should be enough.
 =cut
 
 $arg = 'cfg';
+default_buildcfg( $config{cfg} || $opt{cfg}{dft}, $config{perl_version} );
 $config{ $arg } = prompt_file( $arg );
 check_buildcfg( $config{ $arg } );
 
@@ -721,18 +822,19 @@ SYNCER: {
             $config{ $arg } = prompt( $arg );
         }
         unless ( $config{sfile} ) {
-            $arg = ' snapext';
+            $arg = 'snapext';
             $config{ $arg } = prompt( $arg );
         }
         $arg = 'tar';
         $config{ $arg } = prompt( $arg );
 
         $arg = 'patchup';
-        if ( whereis( 'patch' ) ) {
-            $config{ $arg } = lc prompt( $arg ) eq 'y' ? 1 : 0;
+        $config{patchbin} ||= find_a_patch();
+        if ( $config{patchbin} ) {
+            $config{ $arg } = prompt_yn( $arg );
 
             if ( $config{ $arg } ) {
-                for $arg (qw( pserver pdir unzip patch )) {
+                for $arg (qw( pserver pdir unzip patchbin )) {
                     $config{ $arg } = prompt( $arg );
                 }
                 $opt{cleanup}->{msg} .= " 2(patches) 3(both)";
@@ -788,11 +890,11 @@ There is an issue when using the "forest" sync, but I will look into that.
 =cut
 
 # Is it just my NetBSD-1.5 box with an old patch?
-my $patchbin = whereis( 'gpatch' ) || whereis( 'patch' );
+my $patchbin = find_a_patch();
 PATCHER: {
     last PATCHER unless $patchbin;
-    $config{patch} = $patchbin;
-    print "\nFound [$config{patch}]";
+    $config{patchbin} = $patchbin;
+    print "\nFound [$config{patchbin}]";
     $arg = 'pfile';
     $config{ $arg } = prompt_file( $arg, 1 );
 
@@ -879,6 +981,7 @@ See L<Test::Smoke::Mailer> and L<mailrpt.pl>
 =cut
 
 $arg = 'mail';
+$opt{ $arg }{dft} = ! $options{usedft};
 $config{ $arg } = prompt_yn( $arg );
 MAIL: {
     last MAIL unless $config{mail};
@@ -888,6 +991,9 @@ MAIL: {
     $arg = 'to';
     while ( !$config{ $arg } ) { $config{ $arg } = prompt( $arg ) }
 
+    $arg = 'bcc';
+    $config{ $arg } = prompt( $arg );
+
     MAILER: {
         local $_ = $config{ 'mail_type' };
 
@@ -899,12 +1005,16 @@ MAIL: {
 
         /^(?:Mail::Sendmail|MIME::Lite)$/ && do {
             $arg = 'from';
+            $opt{ $arg }{chk} = '\S+';
             $config{ $arg } = prompt( $arg );
 
             $arg = 'mserver';
             $config{ $arg } = prompt( $arg );
         };
     }
+    $arg = 'ccp5p_onfail';
+    $config{ $arg } = prompt_yn( $arg );
+
     $arg = 'cc';
     $config{ $arg } = prompt( $arg );
 }
@@ -958,6 +1068,25 @@ EO_MSG
     ];
 }
 
+=item make finetuning
+
+Two different config options to accomodate the same thing: 
+I<parallel build> and I<serial testing>
+
+  * makeopt  => used by Test::Smoke::Smoker::_make()
+  * testmake => use a different binary for "make _test"
+
+=cut
+
+$arg = 'makeopt';
+$opt{ $arg }->{dft} = '-nologo' if is_win32 && $config{w32make} =~ /nmake/i;
+$config{ $arg } = prompt( $arg );
+
+unless ( is_win32 ) {
+    $arg = 'testmake';
+    $config{ $arg } = prompt( $arg ) || 'make';
+}
+
 =item umask
 
 C<umask> will be set in the shell-script that starts the smoke.
@@ -1013,8 +1142,8 @@ $config{ $arg } = prompt_yn( $arg );
 When C<< $Config{d_alarm} >> is found we can use C<alarm()> to abort 
 long running smokes. Leave this value empty to keep the old behaviour.
 
-    07:30 => F<mktest.pl> is aborted on 7:30 localtime
-   +23:45 => F<mktest.pl> is aborted after 23 hours and 45 minutes
+    07:30 => F<smokeperl.pl> is aborted on 7:30 localtime
+   +23:45 => F<smokeperl.pl> is aborted after 23 hours and 45 minutes
 
 Thank you Jarkko for donating this suggestion.
 
@@ -1025,6 +1154,20 @@ if ( $Config{d_alarm} ) {
     $config{ $arg } = prompt( $arg );
 }
 
+=item adir
+
+The smokereports are lost after a new SYNCTREE step, it might be handy
+to archive them along with the logfile.
+
+If you want this then set the directory where you want the stored
+(empty value means no archiving).
+
+=cut
+
+$arg = 'adir';
+$config{ $arg } = prompt_dir( $arg );
+$config{lfile} = File::Spec->rel2abs( $options{log}, cwd );
+
 =item schedule stuff
 
 =over 4
@@ -1182,7 +1325,7 @@ sub sort_configkeys {
 
         # Sync related
         qw( sync_type fsync rsync opts source tar server sdir sfile
-            patchup pserver pdir unzip patch cleanup cdir hdir patch pfile ),
+            patchup pserver pdir unzip patchbin cleanup cdir hdir pfile ),
 
         # OS specific make related
         qw( w32args w32cc w32make ),
@@ -1191,7 +1334,13 @@ sub sort_configkeys {
         qw( force_c_locale locale defaultenv ),
 
         # Report related
-        qw( mail mail_type mserver from to cc ),
+        qw( mail mail_type mserver from to ccp5p_onfail cc bcc ),
+
+        # Archive reports and logfile
+        qw( adir lfile ),
+
+        # make fine-tuning
+        qw( makeopt testmake ),
     );
 
     my $i = 0;
@@ -1214,6 +1363,7 @@ C<write_sh()> creates the shell-script.
 sub write_sh {
     my $cwd = cwd();
     my $jcl = "$options{jcl}.sh";
+    my $smokeperl = File::Spec->catfile( $FindBin::Bin, 'smokeperl.pl' );
     my $cronline = schedule_entry( File::Spec->catfile( $cwd, $jcl ), 
                                    $cron, $crontime );
     my $handle_lock = $config{killtime} ? <<EO_CONT : <<EO_DIE;
@@ -1235,7 +1385,7 @@ EO_DIE
 #
 # Written by $0 v$VERSION
 # @{[ scalar localtime ]}
-# NOTE: Changes made in this file wil be \*lost\*
+# NOTE: Changes made in this file will be \*lost\*
 #       after rerunning $0
 #
 # $cronline
@@ -1247,12 +1397,12 @@ continue=''
 if test -f "\$LOCKFILE" && test -s "\$LOCKFILE" ; then
 $handle_lock
 fi
-echo "\$LOCKFILE" > "\$LOCKFILE"
+echo "\$CFGNAME" > "\$LOCKFILE"
 
-PATH=$cwd:$ENV{PATH}
+PATH=$FindBin::Bin:$ENV{PATH}
 export PATH
 umask $config{umask}
-$^X smokeperl.pl -c "\$CFGNAME" \$continue \$\* > $options{log} 2>&1
+$^X $smokeperl -c "\$CFGNAME" \$continue \$\* > $options{log} 2>&1
 
 rm "\$LOCKFILE"
 EO_SH
@@ -1273,7 +1423,10 @@ because it uses commands that are not supported by B<COMMAND.COM>
 
 sub write_bat {
     my $cwd = File::Spec->canonpath( cwd() );
+    my $findbin_bin = File::Spec->canonpath( $FindBin::Bin );
 
+    my $smokeperl  = File::Spec->catfile( $findbin_bin, 'smokeperl.pl' );
+    my $archiverpt = File::Spec->catfile( $findbin_bin, 'archiverpt.pl' );
     my $copycmd = $config{w32args}->[1] ne "BORLAND" ? "" : <<EOCOPYCMD;
 
 REM I found hanging XCOPY while smoking with BORLAND
@@ -1284,6 +1437,9 @@ EOCOPYCMD
     my $jcl = "$options{jcl}.cmd";
     my $atline = schedule_entry( File::Spec->catfile( $cwd, $jcl ), 
                                  $cron, $crontime );
+    my $archive = $config{lfile} 
+        ? qq/$^X $archiverpt -c "\%CFGNAME\%"/ : "";
+
     local *MYSMOKEBAT;
     open MYSMOKEBAT, "> $jcl" or
         die "Cannot write '$jcl': $!";
@@ -1293,28 +1449,29 @@ setlocal
 
 REM Written by $0 v$VERSION
 REM @{[ scalar localtime ]}
-REM NOTE: Changes made in this file wil be \*lost\*
+REM NOTE: Changes made in this file will be \*lost\*
 REM       after rerunning $0
 $copycmd
 REM $atline
 
-set WD=$cwd\
+set WD=$cwd\\
 rem Change drive-Letter
 for \%\%L in ( "\%WD\%" ) do \%\%~dL
 cd "\%WD\%"
 set CFGNAME=$options{config}
 set LOCKFILE=$options{prefix}.lck
 if NOT EXIST \%LOCKFILE\% goto START_SMOKE
-    FIND "\%LOCKFILE\%" \%LOCKFILE\% > NUL:
+    FIND "\%CFGNAME\%" \%LOCKFILE\% > NUL:
     if ERRORLEVEL 1 goto START_SMOKE
     echo We seem to be running [or remove \%LOCKFILE\%]>&2
     goto :EOF
 
 :START_SMOKE
-    echo \%LOCKFILE\% > \%LOCKFILE\%
+    echo \%CFGNAME\% > \%LOCKFILE\%
     set OLD_PATH=\%PATH\%
-    set PATH=$cwd;\%PATH\%
-    $^X smokeperl.pl -c "\%CFGNAME\%" \%* > "\%WD\%\\$options{log}" 2>&1
+    set PATH=$findbin_bin;\%PATH\%
+    $^X $smokeperl -c "\%CFGNAME\%" \%* > "\%WD\%\\$options{log}" 2>&1
+    $archive
     set PATH=\%OLD_PATH\%
 
 del \%LOCKFILE\%
@@ -1327,8 +1484,8 @@ EO_BAT
 }
 
 sub prompt {
-    my( $message, $alt, $df_val, $chk ) = 
-        @{ $opt{ $_[0] } }{qw( msg alt dft chk )};
+    my( $message, $alt, $df_val, $chk, $nck ) = 
+        @{ $opt{ $_[0] } }{qw( msg alt dft chk nck )};
 
     $df_val = $conf->{ $_[0] } if exists $conf->{ $_[0] };
     unless ( defined $message ) {
@@ -1336,8 +1493,9 @@ sub prompt {
         (caller 1)[3] or print "Got [$retval]\n";
         return $df_val;
     }
-    $message =~ s/\s+$//;
 
+    $message =~ s/\s+$//;
+        
     my %ok_val;
     %ok_val = map { (lc $_ => 1) } @$alt if @$alt;
     $chk ||= '.*';
@@ -1351,8 +1509,12 @@ sub prompt {
 
     my( $input, $clear );
     INPUT: {
-        print "$alts\[$default] \$ ";
-        chomp( $input = <STDIN> );
+        if ( $options{usedft} ) {
+            $input = defined $df_val ? $df_val : " ";
+        } else {
+            print "$alts\[$default] \$ ";
+            chomp( $input = <STDIN> );
+        }
         if ( $input eq " " ) {
             $input = "";
             $clear = 1;
@@ -1362,7 +1524,10 @@ sub prompt {
             $input = $df_val unless length $input;
         }
 
-        printf "Input does not match $chk\n" and redo INPUT
+        print "Input should not match: '/$nck/i'\n" and redo INPUT
+            if $nck && $input =~ m/$nck/i;
+        
+        print "Input does not match: 'm/$chk/'\n" and redo INPUT
             unless $input =~ m/$chk/i;
 
         last INPUT unless %ok_val;
@@ -1378,7 +1543,7 @@ sub prompt {
 
 sub prompt_dir {
 
-    if ( exists $conf->{ $_[0] } )  {
+    if ( exists $conf->{ $_[0] } && $conf->{ $_[0] } )  {
         $conf->{ $_[0] } = File::Spec->rel2abs( $conf->{ $_[0] } )
             unless File::Spec->file_name_is_absolute( $conf->{ $_[0] } );
     }
@@ -1387,6 +1552,11 @@ sub prompt_dir {
     
 
         my $dir = prompt( @_ );
+        if ( $dir eq "" && ! @{ $opt{ $_[0] }->{alt} } && 
+             ! $opt{ $_[0] }->{chk} ) {
+            print "Got []\n";
+            return "";
+        }
 
         # thanks to perlfaq5
         $dir =~ s{^ ~ ([^/]*)}
@@ -1465,6 +1635,16 @@ sub whereis {
     return @fnames ? wantarray ? @fnames : \@fnames : '';
 }
 
+sub find_a_patch {
+
+    my $patch_bin;
+    foreach my $patch (qw( gpatch npatch patch )) {
+        $patch_bin = whereis( $patch ) or next;
+        my $version = `$patch_bin --version`;
+        $? or return $patch_bin;
+    }
+}
+
 sub renice {
     my $rn_val = shift;
 
@@ -1489,10 +1669,9 @@ sub get_avail_sync {
 
     my $pversion = $config{perl_version} || '5.9.x';
 
-    # (has_ftp && 5.9.x) || (has_lwp && !5.6.x)
+    # (has_ftp && 5.9.x) || has_lwp
     unshift @synctype, 'snapshot' 
-        if ( $has_ftp && $pversion eq '5.9.x' ) ||
-           ( $has_lwp && $pversion ne '5.6.2' );
+        if ( $has_ftp && $pversion eq '5.9.x' ) || $has_lwp;
     unshift @synctype, 'rsync' if whereis( 'rsync' );
     return @synctype;
 }
@@ -1528,7 +1707,14 @@ sub tar_fmt {
 }
 
 sub check_locale {
-    # I only know one way...
+    # I only know one way... and one for Darwin (perhaps FreeBSD)
+    if ( $^O =~ /darwin|bsd/i ) {
+        local *USL;
+        opendir USL, '/usr/share/locale' or return;
+        my @list = grep /utf-?8$/i => readdir USL;
+        closedir USL;
+        return @list;
+    } 
     my $locale = whereis( 'locale' );
     return unless $locale;
     return grep /utf-?8$/i => split /\n/, `$locale -a`;
@@ -1628,14 +1814,35 @@ sub get_avail_w32compilers {
 }
 
 sub get_Win_version {
-    my @osversion = Win32::GetOSVersion();
-
-    my $win_version = join '.', @osversion[ 1, 2 ];
-    $win_version .= " $osversion[0]" if $osversion[0];
-
+    require Test::Smoke::SysInfo;
+    ( my $win_version = Test::Smoke::SysInfo::__get_os() ) =~ s/^[^-]*- //;
     return $win_version;
 }
 
+=item default_buildcfg( $file_name, $pversion )
+
+Check to see if C<$file_name> exists. If not, copy the default config
+for C<$pversion> to C<$file_name>.
+
+=cut
+
+sub default_buildcfg {
+    my( $file_name, $pversion ) = @_;
+    -f $file_name and return 1;
+
+    $pversion =~ tr/.//d;
+    $pversion eq '59x' and $pversion = 'current';
+    my $basename = $^O eq 'MSWin32'
+        ? "w32current.cfg" : "perl${pversion}.cfg";
+
+    my $dftbcfg = File::Spec->catfile( $FindBin::Bin, $basename );
+    -f $dftbcfg 
+        or die "You seem to have an incomplete Test::Smoke installation" . 
+               "($dftbcfg is missing)!\n";
+    copy $dftbcfg, $file_name
+        and print "\nCreated buildconfig '$file_name'";
+}
+
 =item check_buildcfg
 
 We will try to check the build configurations file to see if we should
@@ -1656,7 +1863,10 @@ sub check_buildcfg {
     close BCFG;
     my $oldcfg = join "", grep !/^#/ => @bcfg;
 
-    my @no_option = ( );
+    my ($rev, @vparts ) = $config{perl_version} =~ /^(\d)(?:\.(\d+))+/;
+    my $pversion = sprintf "%d.%03d%02d", $rev, @vparts, 0;
+    print "Checking '$file_name' ($pversion/$^O)\n";
+    my @no_option = $pversion >= 5.009 ? ( '-Uuseperlio' ) : ( );
     OSCHECK: {
         local $_ = $^O;
         /darwin|bsd/i && do { 
@@ -1709,6 +1919,7 @@ write the new one in its place.
 sub finish_cfgcheck {
     my( $overwrite, $fname, $bcfg ) = @_;
 
+    my $msg = "";
     if ( $overwrite ) {
         my $backup = "$fname.bak";
         -f $backup and chmod( 0775, $backup ) and unlink $backup;
@@ -1716,6 +1927,7 @@ sub finish_cfgcheck {
             warn "Cannot rename '$fname' to '$backup': $!";
     } else {
         $fname = "$options{prefix}.cfg";
+        $msg = " (in case you want it later)";
     }
     # change the filemode (make install used to make perlcurrent.cfg readonly)
     -f $fname and chmod 0775, $fname;
@@ -1728,7 +1940,7 @@ sub finish_cfgcheck {
         warn "Error on close '$fname': $!";
         return;
     };
-    print "Wrote '$fname'\n";
+    print "Wrote '$fname'$msg\n";
 }
 
 =back
@@ -1741,7 +1953,7 @@ Schedule, logfile optional
 
 In case I forget to update the C<$VERSION>:
 
-    $Id: configsmoke.pl 319 2003-08-03 11:24:26Z abeltje $
+    $Id: configsmoke.pl 705 2004-05-04 20:08:07Z abeltje $
 
 =head1 COPYRIGHT
 
