@@ -1,9 +1,10 @@
 package Test::Smoke;
 use strict;
 
-# $Id: Smoke.pm 320 2003-08-03 13:41:09Z abeltje $
-use vars qw( $VERSION $conf @EXPORT );
-$VERSION = '1.18';
+# $Id: Smoke.pm 716 2004-07-24 17:51:09Z abeltje $
+use vars qw( $VERSION $REVISION $conf @EXPORT );
+$VERSION  = '1.19';
+$REVISION = __get_ts_patchlevel();
 
 use base 'Exporter';
 @EXPORT  = qw( $conf &read_config &run_smoke );
@@ -14,7 +15,7 @@ use Test::Smoke::Policy;
 use Test::Smoke::BuildCFG;
 use Test::Smoke::Smoker;
 use Test::Smoke::SourceTree qw( :mani_const );
-use Test::Smoke::Util qw( get_patch );
+use Test::Smoke::Util qw( get_patch skip_config );
 use Config;
 
 =head1 NAME
@@ -28,8 +29,9 @@ Test::Smoke - The Perl core test smoke suite
     use vars qw( $VERSION );
     $VERSION = Test::Smoke->VERSION;
 
-    read_config( $config_name ) or warn Test::Smoke->config_error; 
-    
+    read_config( $config_name ) or die Test::Smoke->config_error; 
+
+    run_smoke;
 
 =head1 DESCRIPTION
 
@@ -86,7 +88,7 @@ sub do_manifest_check {
     my( $ddir, $smoker ) = @_;
 
     my $tree = Test::Smoke::SourceTree->new( $ddir );
-    my $mani_check = $tree->check_MANIFEST( 'mktest.out' );
+    my $mani_check = $tree->check_MANIFEST( 'mktest.out', 'mktest.rpt' );
     foreach my $file ( sort keys %$mani_check ) {
         if ( $mani_check->{ $file } == ST_MISSING ) {
             $smoker->log( "MANIFEST declared '$file' but it is missing\n" );
@@ -96,25 +98,46 @@ sub do_manifest_check {
     }
 }
 
-=item run_smoke( $continue, $patch )
+=item run_smoke( [$continue[, @df_buildopts]] )
 
 C<run_smoke()> sets up de build environment and gets the private Policy
 file and build configurations and then runs the smoke stuff for all 
 configurations.
 
+All arguments after the C<$continue> are taken as default buildoptions
+and passed to C<./Configure>.
+
 =cut
 
 sub run_smoke {
     my $continue = shift;
-    my $patch = shift || Test::Smoke::Util::get_patch( $conf->{ddir} );
+    defined $continue or $continue = $conf->{continue};
+
+    my @df_buildopts = @_ ? grep /^-[DUA]/ => @_ : ();
+    # We *always* want -Dusedevel!
+    push @df_buildopts, '-Dusedevel' 
+        unless grep /^-Dusedevel$/ => @df_buildopts;
+    Test::Smoke::BuildCFG->config( dfopts => join " ", @df_buildopts );
+
+    my $patch = Test::Smoke::Util::get_patch( $conf->{ddir} );
+
+    exists $Config{ldlibpthname} && $Config{ldlibpthname} and
+        $ENV{ $Config{ldlibpthname} } ||= '',
+        substr( $ENV{ $Config{ldlibpthname} }, 0, 0)  =
+            "$conf->{ddir}$Config{path_sep}";
+
+    my $logfile = File::Spec->catfile( $conf->{ddir}, 'mktest.out' );
+    my $BuildCFG = $continue 
+        ? Test::Smoke::BuildCFG->continue( $logfile, $conf->{cfg}, 
+                                           v => $conf->{v} )
+        : Test::Smoke::BuildCFG->new( $conf->{cfg}, v => $conf->{v} );
 
     local *LOG;
     my $mode = $continue ? ">>" : ">";
-    open LOG, "$mode " . File::Spec->catfile( $conf->{ddir}, 'mktest.out' )  or
-        die "Cannot create 'mktest.out': $!";
+    open LOG, "$mode $logfile" or die "Cannot create 'mktest.out': $!";
 
-    my $Policy   = Test::Smoke::Policy->new( File::Spec->updir, $conf->{v} );
-    my $BuildCFG = Test::Smoke::BuildCFG->new( $conf->{cfg}, v => $conf->{v} );
+    my $Policy   = Test::Smoke::Policy->new( File::Spec->updir, $conf->{v},
+                                             $BuildCFG->policy_targets );
 
     my $smoker   = Test::Smoke::Smoker->new( \*LOG, $conf );
     $smoker->mark_in;
@@ -123,6 +146,7 @@ sub run_smoke {
         $smoker->tty( "Running smoke tests without \$ENV{PERLIO}\n" );
 
     unless ( $continue ) {
+        $smoker->_make( "-i distclean 2>/dev/null" );
         $smoker->ttylog( "Smoking patch $patch\n" ); 
         do_manifest_check( $conf->{ddir}, $smoker );
     }
@@ -149,18 +173,24 @@ sub run_smoke {
    };
 }
 
-=item skip_config( $config ) 
+=item __get_ts_patchlevel( )
 
-Returns true if this config should be skipped.
+Read the contents of F<.patch> or use the subversion placeholder
+C<$Rev: 716 $>.
 
 =cut
 
-sub skip_config {
-    my( $config ) = @_;
+use FindBin;
+use File::Spec::Functions;
 
-    my $skip = $config->has_arg(qw( -Uuseperlio -Dusethreads )) ||
-               $config->has_arg(qw( -Uuseperlio -Duseithreads ));
-    return $skip;
+sub __get_ts_patchlevel {
+    my( $rev ) = q$Rev: 716 $ =~ /(\d+)/;
+    my $dotpatch = catfile $FindBin::Bin, '.patch';
+    local *DOTPATCH;
+    open DOTPATCH, "< $dotpatch" or return $rev;
+    chomp( my $plevel = <DOTPATCH> );
+    close DOTPATCH;
+    return $plevel > $rev ? $plevel : $rev;
 }
 
 1;
@@ -169,13 +199,11 @@ sub skip_config {
 
 =head1 REVISION
 
-$Id: Smoke.pm 320 2003-08-03 13:41:09Z abeltje $
+$Id: Smoke.pm 716 2004-07-24 17:51:09Z abeltje $
 
 =head1 COPYRIGHT
 
-(c) 2003, All rights reserved.
-
-  * Abe Timmerman <abeltje@cpan.org>
+(c) 2003-2004, Abe Timmerman <abeltje@cpan.org> All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
