@@ -1,29 +1,33 @@
 package Test::Smoke::Mailer;
 use strict;
 
-# $Id: Mailer.pm 231 2003-07-15 03:10:56Z abeltje $
-use vars qw( $VERSION );
-$VERSION = '0.006';
+# $Id: Mailer.pm 691 2004-04-30 16:24:02Z abeltje $
+use vars qw( $VERSION $P5P $NOCC_RE);
+$VERSION = '0.009';
 
 use Test::Smoke::Util qw( parse_report_Config );
 
+$P5P       = 'perl5-porters@perl.org';
+$NOCC_RE   = ' (?:PASS\b|FAIL\(X\))';
 my %CONFIG = (
-    df_mailer  => 'Mail::Sendmail',
-    df_ddir    => undef,
-    df_v       => 0,
-    df_to      => 'daily-build-reports@perl.org',
-    df_from    => '',
-    df_cc      => '',
-    df_mserver => 'localhost',
+    df_mailer        => 'Mail::Sendmail',
+    df_ddir          => undef,
+    df_v             => 0,
+    df_to            => 'daily-build-reports@perl.org',
+    df_from          => '',
+    df_cc            => '',
+    df_bcc           => '',
+    df_ccp5p_onfail  => 0,
+    df_mserver       => 'localhost',
 
     df_mailbin       => 'mail',
-    mail             => [qw( cc mailbin )],
+    mail             => [qw( bcc cc mailbin )],
     df_mailxbin      => 'mailx',
-    mailx            => [qw( cc mailxbin )],
+    mailx            => [qw( bcc cc mailxbin )],
     df_sendmailbin   => 'sendmail',
-    sendmail         => [qw( from cc sendmailbin )],
-    'Mail::Sendmail' => [qw( from cc mserver )],
-    'MIME::Lite'     => [qw( from cc mserver )],
+    sendmail         => [qw( from bcc cc sendmailbin )],
+    'Mail::Sendmail' => [qw( from bcc cc mserver )],
+    'MIME::Lite'     => [qw( from bcc cc mserver )],
 
     valid_mailer => { sendmail => 1, mail => 1, mailx => 1,
                       'Mail::Sendmail' => 1, 'MIME::Lite' => 1, },
@@ -86,7 +90,7 @@ sub  new {
     my %fields = map {
         my $value = exists $args{$_} ? $args{ $_ } : $CONFIG{ "df_$_" };
         ( $_ => $value )
-    } ( v => ddir => to => @{ $CONFIG{ $mailer } } );
+    } ( v => ddir => to => ccp5p_onfail => @{ $CONFIG{ $mailer } } );
     $fields{ddir} = File::Spec->rel2abs( $fields{ddir} );
 
     DO_NEW: {
@@ -140,6 +144,32 @@ sub error {
     return $self->{error} || '';
 }
 
+=item $self->_get_cc( $subject )
+
+C<_get_cc()> implements the C<--ccp5p_onfail> option. It looks at the
+subject to see if the smoke FAILed and then adds the I<perl5-porters>
+mailing-list to the C<Cc:> field unless it is already part of C<To:>
+or C<Cc:>.
+
+The new behaviour is to only return C<Cc:> on fail. This is determined
+by the new global regex kept in C<< $Test::Smoke::Mailer::NOCC_RE >>.
+
+=cut
+
+sub _get_cc {
+    my( $self, $subject ) = @_;
+    return "" if $subject =~ m/$NOCC_RE/;
+
+    return $self->{cc} || "" unless $self->{ccp5p_onfail};
+
+    my $p5p = $Test::Smoke::Mailer::P5P or return $self->{cc};
+    my @cc = $self->{cc} ? $self->{cc} : ();
+
+    push @cc, $p5p unless $self->{to} =~ /\Q$p5p\E/ || 
+                          $self->{cc} =~ /\Q$p5p\E/;
+    return join ", ", @cc;
+}
+
 =item Test::Smoke::Mailer->config( $key[, $value] )
 
 C<config()> is an interface to the package lexical C<%CONFIG>, 
@@ -217,11 +247,12 @@ sub mail {
     my $self = shift;
 
     my $subject   = $self->fetch_report();
+    my $cc = $self->_get_cc( $subject );
     my $header = "To: $self->{to}\n";
     $header   .= "From: $self->{from}\n" 
         if exists $self->{from} && $self->{from};
-    $header   .= "Cc: $self->{cc}\n" 
-        if exists $self->{cc} && $self->{cc};
+    $header   .= "Cc: $cc\n" if $cc;
+    $header   .= "Bcc: $self->{bcc}\n" if $self->{bcc};
     $header   .= "Subject: $subject\n\n";
 
     $self->{v} > 1 and print "[$self->{sendmailbin} -i -t]\n";
@@ -285,9 +316,11 @@ sub mail {
     my $mailer = $self->{mailbin} || $self->{mailxbin};
 
     my $subject = $self->fetch_report();
+    my $cc = $self->_get_cc( $subject );
 
     my $cmdline = qq|$mailer -s '$subject'|;
-    $cmdline   .= qq| -c '$self->{cc}'| if $self->{cc};
+    $cmdline   .= qq| -c '$cc'| if $cc;
+    $cmdline   .= qq| -b '$self->{bcc}'| if $self->{bcc};
     $cmdline   .= qq| $self->{to}|;
 
     $self->{v} > 1 and print "[$cmdline]\n";
@@ -353,13 +386,15 @@ sub mail {
     $self->{error} = $@ and return undef;
 
     my $subject = $self->fetch_report();
+    my $cc = $self->_get_cc( $subject );
 
     my %message = (
         To      => $self->{to},
         Subject => $subject,
         Body    => $self->{body},
     );
-    $message{cc}   = $self->{cc} if $self->{cc};
+    $message{cc}   = $cc if $cc;
+    $message{bcc}   = $self->{bcc} if $self->{bcc};
     $message{from} = $self->{from} if $self->{from};
     $message{smtp} = $self->{mserver} if $self->{mserver};
 
@@ -422,6 +457,7 @@ sub mail {
     $self->{error} = $@ and return undef;
 
     my $subject = $self->fetch_report();
+    my $cc = $self->_get_cc( $subject );
 
     my %message = (
         To      => $self->{to},
@@ -429,7 +465,8 @@ sub mail {
         Type    => "TEXT",
         Data    => $self->{body},
     );
-    $message{Cc}   = $self->{cc} if $self->{cc};
+    $message{Cc}   = $cc  if $cc;
+    $message{Bcc}   = $self->{bcc} if $self->{bcc};
     $message{From} = $self->{from} if $self->{from};
     MIME::Lite->send( smtp => $self->{mserver} ) if $self->{mserver};
 
