@@ -1,18 +1,18 @@
 package Test::Smoke::Smoker;
 use strict;
 
-# $Id: Smoker.pm 316 2003-08-02 12:28:49Z abeltje $
+# $Id: Smoker.pm 702 2004-05-02 22:47:59Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.004';
+$VERSION = '0.015';
 
 use Cwd;
-use File::Spec;
+use File::Spec::Functions qw( :DEFAULT abs2rel rel2abs );
 use Config;
-use Test::Smoke::Util;
+use Test::Smoke::Util qw( get_smoked_Config skip_filter );
 BEGIN { eval q{ use Time::HiRes qw( time ) } }
 
 my %CONFIG = (
-    df_ddir           => File::Spec->curdir(),
+    df_ddir           => curdir(),
     df_v              => 0,
     df_run            => 1,
     df_fdir           => undef,
@@ -25,6 +25,9 @@ my %CONFIG = (
     df_w32cc          => 'MSVC60',
     df_w32make        => 'nmake',
     df_w32args        => [ ],
+
+    df_makeopt        => "",
+    df_testmake       => 'make',
 );
 
 # Define some constants that we can use for
@@ -193,9 +196,22 @@ sub smoke {
 
     $self->make_distclean;
 
+    $self->{v} > 1 and $self->extra_manicheck;
+
     $self->handle_policy( $policy, $config->policy );
 
-    $self->Configure( $config ) or do {
+    my $c_result = $self->Configure( $config );
+    # Log the compiler info now, the last config could fail
+    { # can we config.sh without Configure success?
+        my %cinfo = get_smoked_Config( $self->{ddir} => qw(
+            cc ccversion gccversion
+        ));
+        my $version = $cinfo{gccversion} || $cinfo{ccversion};
+        $self->log( "\nCompiler info: $cinfo{cc} version $version\n" )
+            if $cinfo{cc};
+    }
+
+    $c_result or do {
         $self->ttylog( "Unable to configure perl in this configuration\n" );
         return 0;
     };
@@ -247,6 +263,31 @@ sub make_distclean {
     }
 }
 
+=item $smoker->extra_manicheck( )
+
+C<extra_manicheck()> will only work for C<< $self->{v} > 1 >> and does
+an extra integrity check comparing F<MANIFEST> and the
+source-tree. Output is send to the tty.
+
+=cut
+
+sub extra_manicheck {
+    my $self = shift;
+    $self->{v} > 1 or return;
+
+    require Test::Smoke::SourceTree;
+    Test::Smoke::SourceTree->import( qw( :mani_const ) );
+    my $tree = Test::Smoke::SourceTree->new( $self->{ddir} );
+    my $mani_check = $tree->check_MANIFEST(qw( mktest.out mktest.rpt ));
+    foreach my $file ( sort keys %$mani_check ) {
+        if ( $mani_check->{ $file } == ST_MISSING() ) {
+            $self->tty( "manicheck: missing '$file' (not in source-tree)\n" );
+        } elsif ( $mani_check->{ $file } == ST_UNDECLARED() ) {
+            $self->tty( "manicheck: extra '$file' (not in MANIFEST)\n" );
+        }
+    }
+}
+
 =item $smoker->handle_policy( $policy, @substs );
 
 C<handle_policy()> will try to apply the substition rules and then 
@@ -308,12 +349,22 @@ sub make_ {
 
     $self->tty( "\nmake ..." );
     $self->_make( "" );
+    if ( $self->{is_win32} ) { # Win32 creates config.sh during make
+        my %cinfo = get_smoked_Config( $self->{ddir} => qw(
+            cc ccversion gccversion
+        ));
+        my $version = $cinfo{gccversion} || $cinfo{ccversion};
+        $self->log( "\nCompiler info: $cinfo{cc} version $version\n" )
+            if $cinfo{cc};
+    }
 
     my $exe_ext  = $Config{_exe} || $Config{exe_ext};
     my $miniperl = "miniperl$exe_ext";
     my $perl     = "perl$exe_ext";
     -x $miniperl or return BUILD_NOTHING;
-    return -x $perl ? BUILD_PERL : BUILD_MINIPERL;
+    return -x $perl 
+        ? $self->{_run_exit} ? BUILD_MINIPERL : BUILD_PERL
+        : BUILD_MINIPERL;
 }
 
 =item make_test_prep( )
@@ -326,7 +377,7 @@ sub make_test_prep {
     my $self = shift;
 
     my $exe_ext = $Config{_exe} || $Config{exe_ext};
-    my $perl = File::Spec->catfile( "t", "perl$exe_ext" );
+    my $perl = catfile( "t", "perl$exe_ext" );
 
     $self->{run} and unlink $perl;
     $self->_make( "test-prep" );
@@ -372,14 +423,14 @@ sub make_test {
             $ENV{LC_ALL} = $self->{locale};
             $perlio_logmsg .= ":$self->{locale}";
         }
-        $self->ttylog( "PERLIO = $perlio_logmsg\t" );
+        $self->ttylog( "TSTENV = $perlio_logmsg\t" );
 
         unless ( $self->{run} ) {
-            $self->ttylog( "bailing out ...\n" );
+            $self->ttylog( "bailing out (--norun)...\n" );
             next;
 	}
 
-        my $test_target = $self->{is56x} ? 'test' : '_test';
+        my $test_target = $self->{is56x} ? 'test-notty' : '_test';
         local *TST;
         # MSWin32 builds from its own directory
         if ( $self->{is_win32} ) {
@@ -389,7 +440,7 @@ sub make_test {
             chdir ".." or die "unable to chdir () out of 'win32'";
         } else {
             local $ENV{PERL} = "./perl";
-            open TST, "make $test_target |" or do {
+            open TST, "$self->{testmake} $test_target |" or do {
                 use Carp;
                 Carp::carp "Cannot fork 'make _test': $!";
                 next;
@@ -399,11 +450,11 @@ sub make_test {
         my @nok = ();
         select ((select (TST), $| = 1)[0]);
         while (<TST>) {
-            $self->{v} > 2 and $self->tty( $_ );
+            $self->{v} > 1 and $self->tty( $_ );
             skip_filter( $_ ) and next;
 
             # make mkovz.pl's life easier
-            s/(.)(PERLIO\s+=\s+\w+)/$1\n$2/;
+            s/(.)(TSTENV\s+=\s+\w+)/$1\n$2/;
 
             if (m/^u=.*tests=/) {
                 s/(\d\.\d*) /sprintf "%.2f ", $1/ge;
@@ -420,7 +471,7 @@ sub make_test {
         };
 #        $self->log( map { "    $_" } @nok );
         if (grep m/^All tests successful/, @nok) {
-            $self->log( "All tests successful\n" );
+            $self->log( "All tests successful.\n" );
             $self->tty( "\nOK, archive results ..." );
             $self->{patch} and $nok[0] =~ s/\./ for .patch = $self->{patch}./;
         } else {
@@ -438,49 +489,56 @@ sub make_test {
 =cut
 
 sub extend_with_harness {
-    my( $self, @nok ) = @_;
-    my @harness;
-    for ( @nok ) {
-        m!^(?:\.\.[\\/])?(\w+/[-\w/]+).*! or next;
-        # Remeber, we chdir into t, so -f is false for op/*.t etc
-        my $test_name = "$1.t";
-        push @harness, (-f $test_name) 
-            ? File::Spec->catdir( File::Spec->updir, $test_name )
-            : $test_name;
-    }
+    my $self = shift;
+    my %inconsistent = $self->_transform_testnames( @_ );
+    my @harness = sort keys %inconsistent;
     if ( @harness ) {
         local $ENV{PERL_SKIP_TTY_TEST} = 1;
-	        $self->tty( "\nExtending failures with Harness\n" );
-        my $harness = $self->{is_win32} ?
-        join " ", map { 
-            s{^\.\.[/\\]}{};
-	            m/^(?:lib|ext)/ and $_ = "../$_";
-            $_;
-        } @harness : "@harness";
+        my $harness = join " ", @harness;
+        $self->tty( "\nExtending failures with harness:\n\t$harness\n" );
         my $changed_dir;
         chdir 't' and $changed_dir = 1;
         my $harness_all_ok = 0;
+        my $tst_perl = catfile( curdir(), 'perl' );
         my $harness_out = join "", map {
             my( $name, $fail ) = 
                 m/(\S+\.t)\s+.+%\s+([\d?]+(?:[-\s]+\d+)*)/;
             if ( $name ) {
-                my $dots = '.' x (30 - length $name );
+                delete $inconsistent{ $name };
+                my $dots = '.' x (40 - length $name );
                 "    $name${dots}FAILED $fail\n";
             } else {
                 ( $fail ) = m/^\s+(\d+(?:[-\s]+\d+)*)/;
-                " " x 41 . "$fail\n";
+                " " x 51 . "$fail\n";
             }
-        } grep m/^\s+\d+(?:[-\s]+\d+)/ ||
+        } grep m/^\s+\d+(?:[-\s]+\d+)*/ ||
                m/\S+\.t\s+.+%\s+[\d?]+(?:[-\s+]\d+)*/ => map {
             /All tests successful/ && $harness_all_ok++;
             $self->{v} > 1 and $self->tty( $_ );
             $_;
-        } $self->_run( "./perl harness $harness" );
+        } $self->_run( "$tst_perl harness $harness" );
+        # safeguard against empty results
+        $inconsistent{ $_ } ||= 'FAILED' for keys %inconsistent;
         $harness_out =~ s/^\s*$//;
-        $harness_out ||= join "", map "    $_" => @nok
-            unless $harness_all_ok;
+        if ( $harness_all_ok ) {
+            $harness_out .= scalar keys %inconsistent
+                ? "Inconsistent test results (between TEST and harness):\n" . 
+                  join "", map {
+                      my $dots = '.' x (40 - length $_ );
+                      "    $_${dots}$inconsistent{ $_ }\n";
+                  } keys %inconsistent
+                : $harness_out ? "" : "All tests successful.";
+        } else {
+            $harness_out .= scalar keys %inconsistent
+                ? "Inconsistent test results (between TEST and harness):\n" . 
+                  join "", map {
+                      my $dots = '.' x (40 - length $_ );
+                      "    $_${dots}$inconsistent{ $_ }\n";
+                  } keys %inconsistent
+                : "";
+        }
         $self->ttylog("\n", $harness_out, "\n" );
-        $changed_dir and chdir File::Spec->updir;
+        $changed_dir and chdir updir();
     }
 }
 
@@ -494,7 +552,7 @@ I<miniperl>, so we do C<< S<make minitest> >>.
 sub make_minitest {
     my $self = shift;
 
-    $self->ttylog( "PERLIO = minitest\t" );
+    $self->ttylog( "TSTENV = minitest\t" );
     local *TST;
     # MSWin32 builds from its own directory
     if ( $self->{is_win32} ) {
@@ -538,9 +596,40 @@ sub make_minitest {
     return 1;
 }
 
+=item $self->_trasnaform_testnames( @notok )
+
+C<_transform_testnames()> takes a list of testnames, as found by
+C<TEST> (testname without C<.t> suffix followed by dots and a reason)
+and returns a hash with the filenames relative to the C<t/> directory
+as keys and the reason as value.
+
+=cut
+
+sub _transform_testnames {
+    my( $self, @notok ) = @_;
+    my %inconsistent;
+    for my $nok ( @notok ) {
+        $nok =~ m!^(?:\.\.[\\/])?(\w+[\\/][-\w/\\]+)\.*(.*)! or next;
+        my( $test_name, $status ) = ( $1, $2 );
+        $test_name .= '.t';
+
+        $test_name = $test_name =~ /^(?:ext|lib|t)\b/
+            ? catfile( updir(), $test_name )
+            : catfile( updir(), 't', $test_name );
+
+        my $test_base = catdir( $self->{ddir}, 't' );
+        $test_name = rel2abs( $test_name, $test_base );
+
+        my $test_path = abs2rel( $test_name, $test_base );
+        $test_path =~ tr!\\!/! if $self->{is_win32};
+        $inconsistent{ $test_path } ||= $status;
+    }
+    return %inconsistent;
+}
+
 =item $self->_run( $command[, $sub[, @args]] )
 
-C<run()> returns C<< qx( $command ) >> unless C<$sub> is specified.
+C<_run()> returns C<< qx( $command ) >> unless C<$sub> is specified.
 If C<$sub> is defined (and a coderef) C<< $sub->( $command, @args ) >> will
 be called.
 
@@ -552,7 +641,9 @@ sub _run {
 
     defined $sub and return &$sub( $command, @args );
 
-    return qx( $command );
+    my @output = qx( $command );
+    $self->{_run_exit} = $? >> 8;
+    return wantarray ? @output : join " ", @output;
 }
 
 =item $self->_make( $command )
@@ -565,6 +656,7 @@ stuff to help MSWin32 (the right maker, the directory).
 sub _make {
     my $self = shift;
     my $cmd = shift;
+    $self->{makeopt} and $cmd = "$self->{makeopt} $cmd";
 
     $self->{is_win32} or return $self->_run( "make $cmd" );
 
