@@ -1,9 +1,9 @@
 package Test::Smoke::Util;
 use strict;
 
-# $Id: Util.pm 317 2003-08-03 01:15:09Z abeltje $
+# $Id: Util.pm 712 2004-07-23 10:49:04Z abeltje $
 use vars qw( $VERSION @EXPORT @EXPORT_OK );
-$VERSION = '0.21';
+$VERSION = '0.27';
 
 use base 'Exporter';
 @EXPORT = qw( 
@@ -11,13 +11,13 @@ use base 'Exporter';
     &get_cfg_filename &get_config
     &check_MANIFEST
     &get_patch
-    &skip_filter
+    &skip_config &skip_filter
 );
 
 @EXPORT_OK = qw( 
     &get_ncpu &get_smoked_Config &parse_report_Config 
     &get_regen_headers &run_regen_headers
-    &calc_timeout 
+    &calc_timeout &time_in_hhmm
     &do_pod2usage
 );
 
@@ -122,6 +122,10 @@ installation.
 Set the base directory for the C compiler.
 B<$(CCHOME)\bin> still needs to be in the path!
 
+=item * B<-DIS_WIN95>
+
+sets IS_WIN95 to 'define' to indicate this is Win9[58]
+
 =item * B<-DCRYPT_SRC=...>
 
 The file to use as source for des_fcrypt()
@@ -151,7 +155,9 @@ my %win32_makefile_map = (
 
 sub Configure_win32 {
     my($command, $win32_maker, @args ) = @_;
-    $win32_maker ||= 'nmake';
+    $win32_maker ||= 'nmake'; $win32_maker = lc $win32_maker;
+    my $is_dmake = $win32_maker eq 'dmake';
+    my $is_nmake = $win32_maker eq 'nmake';
 
     local $_;
     my %opt_map = (
@@ -172,6 +178,7 @@ sub Configure_win32 {
         "-Dgcc_v3_2"            => "USE_GCC_V3_2",
         "-Dbccold"              => "BCCOLD",
         "-DCCHOME"              => "CCHOME",
+        "-DIS_WIN95"            => "IS_WIN95",
         "-DCRYPT_SRC"           => "CRYPT_SRC",
         "-DCRYPT_LIB"           => "CRYPT_LIB",
     );
@@ -197,11 +204,12 @@ sub Configure_win32 {
         USE_GCC_V3_2    => 0,
         BCCOLD          => 0,
         CCHOME          => undef,
+        IS_WIN95        => 0,
         CRYPT_SRC       => undef,
         CRYPT_LIB       => undef,
     );
-#    my $def_re = qr/((?:(?:PERL|USE)_\w+)|BCCOLD)/;
-    my $def_re = '((?:(?:PERL|USE)_\w+)|BCCOLD)';
+#    my $def_re = qr/((?:(?:PERL|USE|IS)_\w+)|BCCOLD)/;
+    my $def_re = '((?:(?:PERL|USE|IS)_\w+)|BCCOLD)';
     my @w32_opts = grep ! /^$def_re/, keys %opts;
     my $config_args = join " ", 
         grep /^-D[a-z_]+/, quotewords( '\s+', 1, $command );
@@ -212,7 +220,7 @@ sub Configure_win32 {
     foreach ( quotewords( '\s+', 1, $1) ) {
 	m/^-[des]{1,3}$/ and next;
 	m/^-Dusedevel$/  and next;
-        if ( /^-Accflags=(['"])?(.+)\1/ ) { #emacs' syntaxhighlite
+        if ( /^-Accflags=(['"]?)(.+)\1/ ) { #emacs' syntaxhighlite
            push @buildopt, $2;
            next;
         }
@@ -242,8 +250,10 @@ sub Configure_win32 {
     my $donot_change = 0;
     while (<ORG>) {
         if ( $donot_change ) {
+            # need to help the Win95 build
+            $is_dmake and s/\b$win32_makefile_map{ $win32_maker }\b/smoke.mk/;
             if (m/^\s*CFG_VARS\s*=/) {
-                my( $extra_char, $quote ) = $win32_maker =~ /\bnmake\b/ 
+                my( $extra_char, $quote ) = $is_nmake
                     ? ( "\t", '"' ) : ("~", "" );
                 $_ .= join "", map "\t\t$quote$_$quote\t${extra_char}\t\\\n", 
                                    grep /\w+=/, @args;
@@ -544,14 +554,14 @@ sub get_ncpu {
             last OS_CHECK;
         };
 
-        /hp-ux/i && do {
+        /hp-?ux/i && do {
             my @output = grep /^processor/ => `ioscan -fnkC processor`;
             $cpus = scalar @output;
             last OS_CHECK;
 	};
 
         /irix/i && do {
-            my @output = `hinv -c processor`;
+            my @output = grep /\s+processors?$/i => `hinv -c processor`;
             $cpus = (split " ", $output[0])[0];
             last OS_CHECK;
         };
@@ -598,8 +608,8 @@ fallback information from C<POSIX::uname()> and F<patchlevel.h>.
 =cut
 
 sub get_smoked_Config {
-    my $dir = shift;
-    my %Config = map { ( lc $_ => "" ) } @_;
+    my( $dir, @fields ) = @_;
+    my %Config = map { ( lc $_ => undef ) } @fields;
 
     my $perl_Config_pm = File::Spec->catfile ($dir, "lib", "Config.pm");
     my $perl_config_sh = File::Spec->catfile( $dir, 'config.sh' );
@@ -607,28 +617,40 @@ sub get_smoked_Config {
     if ( open CONF, "< $perl_Config_pm" ) {
 
         while (<CONF>) {
-            if (m/^(our|my) \$[cC]onfig_[sS][hH](.*) = <<'!END!';/..m/^!END!/){
+            if ( m/^(?:
+                       (?:our|my)\ \$[cC]onfig_[sS][hH].*
+                    |
+                       \$_
+                    )\ =\ <<'!END!';/x..m/^!END!/){
                 m/!END!(?:';)?$/      and next;
                 m/^([^=]+)='([^']*)'/ or next;
                 exists $Config{lc $1} and $Config{lc $1} = $2;
             }
         }
         close CONF;
-    } elsif ( open CONF, "< $perl_config_sh" ) {
+    }
+    my %conf2 = map {
+        ( $_ => undef )
+    } grep !defined $Config{ $_ } => keys %Config;
+    if ( open CONF, "< $perl_config_sh" ) {
         while ( <CONF> ) {
             m/^([^=]+)='([^']*)'/ or next; # '
-            exists $Config{ lc $1} and $Config{ lc $1 } = $2;
+            exists $conf2{ $1} and $Config{ lc $1 } = $2;
         }
         close CONF;
-    } else { 
+    }
+    %conf2 = map {
+        ( $_ => undef )
+    } grep !defined $Config{ $_ } => keys %Config;
+    if ( keys %conf2 ) { 
         # Fall-back values from POSIX::uname() (not reliable)
         require POSIX;
         my( $osname, undef, $osvers, undef, $arch) = POSIX::uname();
-        $Config{osname} = lc $osname if exists $Config{osname};
-        $Config{osvers} = lc $osvers if exists $Config{osvers};
-        $Config{archname} = lc $arch if exists $Config{archname};
-        $Config{version} = version_from_patchlevel_h( $dir )
-            if exists $Config{version};
+        $Config{osname}   = lc $osname if exists $conf2{osname};
+        $Config{osvers}   = lc $osvers if exists $conf2{osvers};
+        $Config{archname} = lc $arch   if exists $conf2{archname};
+        $Config{version}  = version_from_patchlevel_h( $dir )
+            if exists $conf2{version};
     }
 
     return %Config;
@@ -649,10 +671,10 @@ sub parse_report_Config {
     my $version  = $report =~ /^Automated.*for (.+) patch/ ? $1 : '';
     my $plevel   = $report =~ /^Automated.*patch (\d+(?:\.\d+\.\d+-RC\d+)?)/
         ? $1 : '';
-    my $osname   = $report =~ /\bon (.*) - / ? $1 : '';
-    my $osvers   = $report =~ /\bon .* - (.*)/? $1 : '';
+    my $osname   = $report =~ /\bon\s+(.*) - / ? $1 : '';
+    my $osvers   = $report =~ /\bon\s+.* - (.*)/? $1 : '';
     $osvers =~ s/\s+\(.*//;
-    my $archname = $report =~ / \((.*)\)/ ? $1 : '';
+    my $archname = $report =~ /:.* \((.*)\)/ ? $1 : '';
     my $summary  = $report =~ /^Summary: (.*)/m ? $1 : '';
 
     return ( $version, $plevel, $osname, $osvers, $archname, $summary );
@@ -663,7 +685,7 @@ sub parse_report_Config {
 C<get_regen_headers()> looks in C<$ddir> to find either 
 F<regen_headers.pl> or F<regen.pl> (change 18851).
 
-Returns undef if not found or a string like C<< "$^X $regen_headers_pl" >>
+Returns undef if not found or a string like C<< $^X "$regen_headers_pl" >>
 
 =cut
 
@@ -673,10 +695,11 @@ sub get_regen_headers {
     $ddir ||= File::Spec->curdir; # Don't smoke in a dir "0"!
 
     my $regen_headers_pl = File::Spec->catfile( $ddir, "regen_headers.pl" );
-    -f $regen_headers_pl and return "$^X $regen_headers_pl";
+
+    -f $regen_headers_pl and return qq[$^X "$regen_headers_pl"];
 
     $regen_headers_pl = File::Spec->catfile( $ddir, "regen.pl" );
-    -f $regen_headers_pl and return "$^X $regen_headers_pl";
+    -f $regen_headers_pl and return qq[$^X "$regen_headers_pl"];
 
     return; # Should this be "make regen_headers"?
 }
@@ -713,7 +736,7 @@ sub run_regen_headers {
     return 1;
 }
 
-=item calc_timeout( $killtime )
+=item calc_timeout( $killtime[, $from] )
 
 C<calc_timeout()> calculates the timeout in seconds. 
 C<$killtime> can be one of two formats:
@@ -727,30 +750,62 @@ to translate that to seconds.
 
 =item B<hh:mm>
 
-This format represents a clock time (localtime).
-Calculate minutes from midnight for both C<$killtime> and C<localtime()>,
-and get the difference.
+This format represents a clock time (localtime).  Calculate minutes
+from midnight for both C<$killtime> and C<localtime($from)>, and get
+the difference. If C<$from> is omitted, C<time()> is used.
+
+If C<$killtime> is the actual time, the timeout will be 24 hours!
 
 =back
 
 =cut
 
 sub calc_timeout {
-    my( $killtime ) = @_;
+    my( $killtime, $from ) = @_;
     my $timeout = 0;
     if ( $killtime =~ /^\+(\d+):([0-5]?[0-9])$/ ) {
         $timeout = 60 * (60 * $1 + $2 );
     } elsif ( $killtime =~ /^((?:[0-1]?[0-9])|(?:2[0-3])):([0-5]?[0-9])$/ ) {
+        defined $from or $from = time;
         my $time_min = 60 * $1 + $2;
-        my( $now_m, $now_h ) = (localtime)[1, 2];
+        my( $now_m, $now_h ) = (localtime $from)[1, 2];
         my $now_min = 60 * $now_h + $now_m;
         my $kill_min = $time_min - $now_min;
-        $kill_min += 60 * 24 if $kill_min < 0;
+        $kill_min += 60 * 24 if $kill_min <= 0;
         $timeout = 60 * $kill_min;
     }
     return $timeout;
 }
 
+=item time_in_hhmm( $diff )
+
+Create a string telling elapsed time in days, hours, minutes, seconds
+from the number of seconds.
+
+=cut
+
+sub time_in_hhmm {
+    my $diff = shift;
+
+    # Only show decimal point for diffs < 5 minutes
+    my $digits = $diff =~ /\./ ? $diff < 5*60 ? 3 : 0 : 0;
+    my $days = int( $diff / (24*60*60) );
+    $diff -= 24*60*60 * $days;
+    my $hour = int( $diff / (60*60) );
+    $diff -= 60*60 * $hour;
+    my $mins = int( $diff / 60 );
+    $diff -=  60 * $mins;
+    $diff = sprintf "%.${digits}f", $diff;
+
+    my @parts;
+    $days and push @parts, sprintf "%d day%s",   $days, $days == 1 ? "" : 's';
+    $hour and push @parts, sprintf "%d hour%s",  $hour, $hour == 1 ? "" : 's';
+    $mins and push @parts, sprintf "%d minute%s",$mins, $mins == 1 ? "" : 's';
+    $diff && !$days && !$hour and push @parts, "$diff seconds";
+
+    return join " ", @parts;
+}
+
 =item do_pod2man( %pod2usage_options )
 
 If L<Pod::Usage> is there then call its C<pod2usage()>.
@@ -779,6 +834,21 @@ EO_MSG
     }
 }
 
+=item skip_config( $config ) 
+
+Returns true if this config should be skipped.
+C<$config> should be a B<Test::Smoke::BuildCFG::Config> object.
+
+=cut
+
+sub skip_config {
+    my( $config ) = @_;
+
+    my $skip = $config->has_arg(qw( -Uuseperlio -Dusethreads )) ||
+               $config->has_arg(qw( -Uuseperlio -Duseithreads ));
+    return $skip;
+}
+
 =item skip_filter( $line )
 
 C<skip_filter()> returns true if the filter rules apply to C<$line>.
