@@ -2,19 +2,15 @@
 use strict;
 $| = 1;
 
-# $Id$
+# $Id: mailrpt.pl 691 2004-04-30 16:24:02Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.017';
+$VERSION = '0.013';
 
 use Cwd;
 use File::Spec;
-my $findbin;
-use File::Basename;
-BEGIN { $findbin = File::Spec->rel2abs( dirname $0 ); }
-use lib File::Spec->catdir( $findbin, 'lib' );
-use lib File::Spec->catdir( $findbin, 'lib', 'inc' );
-use lib $findbin;
-use lib File::Spec->catdir( $findbin, 'inc' );
+use FindBin;
+use lib File::Spec->catdir( $FindBin::Bin, 'lib' );
+use lib $FindBin::Bin;
 use Test::Smoke::Reporter;
 use Test::Smoke::Mailer;
 use Test::Smoke;
@@ -29,12 +25,8 @@ my %opt = (
     ccp5p_onfail => undef,
     from         => undef,
     mserver      => undef,
-    msport       => 25,
-    msuser       => undef,
-    mspass       => undef,
     v            => undef,
 
-    rptfile      => 'mktest.rpt',
     mail         => 1,
     report       => undef,
     defaultenv   => undef,
@@ -45,7 +37,7 @@ my %opt = (
 
 my $defaults = Test::Smoke::Mailer->config( 'all_defaults' );
 
-my %valid_type = map { $_ => 1 } qw( mail mailx sendmail sendemail
+my %valid_type = map { $_ => 1 } qw( mail mailx sendmail 
                                      Mail::Sendmail MIME::Lite );
 
 =head1 NAME
@@ -79,8 +71,7 @@ Other options can override the settings from the configuration file.
     --to <emailaddresses>    Comma separated list (smokers-reports@perl.org)
     --cc <emailaddresses>    Comma separated list
 
-    -t | --type <type>       mail mailx sendmail sendemail Mail::Sendmail
-                             [mandatory]
+    -t | --type <type>       mail mailx sendmail Mail::Sendmail [mandatory]
 
     --nomail                 Don't send the message
     --report                 Create a report anyway
@@ -100,13 +91,6 @@ no extra options
 
     --from <address>
 
-=item * B<options for> -t sendemail
-
-    --from <address>
-    --mserver <smtpserver>  (localhost)
-    --muser <smtpserverusername>
-    --mpass <smtpserverpassword>
-
 =item * B<options for> -t Mail::Sendmail | MIME::Lite
 
     --from <address>
@@ -125,11 +109,11 @@ GetOptions( \%opt,
     'type|t=s', 'ddir|d=s', 'to=s', 'cc=s', 'bcc=s', 'ccp5p_onfail!',
     'v|verbose=i',
 
-    'from=s', 'mserver=s', 'msport=i', 'msuser=s', 'mspass=s',
+    'from=s', 'mserver=s',
 
     'help|h', 'man',
 
-    'config|c:s', 'rptfile|r=s',
+    'config|c:s',
 
     'mail|email!', 'report!', 'defaultenv!',
 ) or do_pod2usage( verbose => 1, myusage => $my_usage );
@@ -140,7 +124,7 @@ $opt{help} and do_pod2usage( verbose => 1, exitval => 0, myusage => $my_usage);
 if ( defined $opt{config} ) {
     $opt{config} eq "" and $opt{config} = 'smokecurrent_config';
     read_config( $opt{config} ) or do {
-        my $config_name = File::Spec->catfile( $findbin, $opt{config} );
+        my $config_name = File::Spec->catfile( $FindBin::Bin, $opt{config} );
         read_config( $config_name );
     };
 
@@ -164,15 +148,13 @@ foreach( keys %$defaults ) {
     $opt{ $_ } = defined $conf->{ $_ } ? $conf->{ $_ } : $defaults->{ $_ };
 }
 
-if ( $opt{mail} ) {
-    exists $valid_type{ $opt{type} } or do_pod2usage( verbose => 0 );
-}
+exists $valid_type{ $opt{type} } or do_pod2usage( verbose => 0 );
 
 $opt{ddir} && -d $opt{ddir} or do_pod2usage( verbose => 0 );
 
-my $cont = check_for_report();
+check_for_report();
 
-if ( $cont && $opt{mail} ) {
+if ( $opt{mail} ) {
     my $mailer = Test::Smoke::Mailer->new( $opt{type} => \%opt );
     $mailer->mail;
 }
@@ -180,26 +162,20 @@ if ( $cont && $opt{mail} ) {
 # Basically: call mkovz.pl unless -f <builddir>/mktest.rpt
 sub check_for_report {
 
-    my $report = File::Spec->catfile( $opt{ddir}, $opt{rptfile} );
+    my $report = File::Spec->catfile( $opt{ddir}, 'mktest.rpt' );
 
     if ( -f $report ) {
         $opt{v} and print "Found [$report]\n";
-        $opt{report} or return 1;
+        $opt{report} or return;
     } else {
         $opt{v} and print "No report found in [$opt{ddir}].\n";
     }
 
     my $reporter = Test::Smoke::Reporter->new( $conf );
-    if ( defined $reporter->{_outfile} ) {
-        $reporter->write_to_file;
-    
-        unless ( -f $report ) {
-            die "Hmmm... cannot find [$report]";
-        }
-        return 1;
-    } else {
-        $opt{v} and print "Skipped, no .out-file\n";
-        return;
+    $reporter->write_to_file;
+
+    unless ( -f $report ) {
+        die "Hmmm... cannot find [$report]";
     }
 }
 
