@@ -2,12 +2,14 @@
 use strict;
 $|=1;
 
-# $Id: smokeperl.pl 313 2003-08-02 01:31:57Z abeltje $
+# $Id: smokeperl.pl 662 2004-03-26 08:29:59Z abeltje $
 use vars qw( $VERSION );
 $VERSION = Test::Smoke->VERSION;
 
 use Cwd;
 use File::Spec;
+use File::Path;
+use File::Copy;
 use FindBin;
 use lib File::Spec->catdir( $FindBin::Bin, 'lib' );
 use lib $FindBin::Bin;
@@ -15,27 +17,33 @@ use Config;
 use Test::Smoke::Syncer;
 use Test::Smoke::Patcher;
 use Test::Smoke;
+use Test::Smoke::Reporter;
 use Test::Smoke::Mailer;
 use Test::Smoke::Util qw( get_patch calc_timeout do_pod2usage );
 
 use Getopt::Long;
-my %options = ( config => 'smokecurrent_config', run => 1,
-                fetch => 1, patch => 1, mail => undef, 
-                continue => 0,
+Getopt::Long::Configure( 'pass_through' );
+my %options = ( config => 'smokecurrent_config', run => 1, pfile => undef,
+                fetch => 1, patch => 1, mail => undef, archive => undef,
+                continue => 0, ccp5p_onfail => undef, killtime => undef,
                 is56x => undef, defaultenv => undef, smartsmoke => undef );
 
 my $myusage = "Usage: $0 [-c configname]";
 GetOptions( \%options, 
     'config|c=s', 
     'fetch!', 
-    'patch!', 
+    'patch!',
+    'ccp5p_onfail!',
     'mail!',
     'run!',
+    'archive!',
     'is56x',
     'defaultenv!',
     'continue',
     'smartsmoke!',
     'snapshot|s=i',
+    'killtime=s',
+    'pfile=s',
 
     'help|h', 'man',
 ) or do_pod2usage(  verbose => 1, myusage => $myusage );
@@ -65,12 +73,19 @@ It can take these options
   --nofetch                Skip the synctree step
   --nopatch                Skip the patch step
   --nomail                 Skip the mail step
+  --noarchive              Skip the archive step (if applicable)
+  --[no]ccp5p_onfail       Do (not) send failure reports to perl5-porters
 
   --continue               Try to continue an interrupted smoke
   --is56x                  This is a perl-5.6.x smoke
   --defaultenv             Run a smoke in the default environment
   --[no]smartsmoke         Don't smoke unless patchlevel changed
   --snapshot <patchlevel>  Set a new patchlevel for snapshot smokes
+  --killtime (+)hh::mm     (Re)set the guard-time for this smoke
+
+  --pfile <patchesfile>    Set a patches-to-apply-file
+
+All other arguments are passed to F<Configure>!
 
 =head1 DESCRIPTION
 
@@ -90,22 +105,18 @@ defined Test::Smoke->config_error and
 
 # Correction for backward compatability
 !defined $options{ $_ } && !exists $conf->{ $_ } and $options{ $_ } = 1
-    for qw( run fetch patch mail );
+    for qw( run fetch patch mail archive );
+
 # Make command-line options override configfile
 defined $options{ $_ } and $conf->{ $_ } = $options{ $_ }
-    for qw( is56x defaultenv smartsmoke run fetch patch mail );
+    for qw( is56x defaultenv continue killtime pfile
+            smartsmoke run fetch patch mail ccp5p_onfail archive );
+
+# Make sure the --pfile command-line override works
+$options{pfile} and $conf->{patch_type} ||= 'multi';
 
 if ( $options{continue} ) {
     $options{v} and print "Will try to continue current smoke\n";
-    my $cfg = Test::Smoke::BuildCFG->new( $conf->{cfg}, v => $conf->{v} );
-    my @found = configs_from_log( $conf->{ddir} );
-    my %found = map { ( $_ => 1 ) } @found;
-    my @pass;
-    foreach my $config ( $cfg->configurations ) {
-        push @pass, $config unless exists $found{ "$config" } ||
-                                   Test::Smoke::skip_config( $config );
-    }
-    $conf->{cfg} = { _list => \@pass };
 } else {
     synctree();
     patchtree();
@@ -114,9 +125,10 @@ if ( $options{continue} ) {
 my $cwd = cwd();
 chdir $conf->{ddir} or die "Cannot chdir($conf->{ddir}): $!";
 call_mktest();
-call_mkovz();
+genrpt();
 mailrpt();
 chdir $cwd;
+archiverpt();
 
 sub synctree {
     my $was_patchlevel = get_patch( $conf->{ddir} ) || -1;
@@ -179,23 +191,19 @@ sub call_mktest {
     }
     $timeout and local $SIG{ALRM} = sub {
         warn "This smoke is aborted ($conf->{killtime})\n";
-        call_mkovz();
+        genrpt();
         mailrpt();
         exit(42);
     };
     $Config{d_alarm} and alarm $timeout;
 
-    run_smoke();
+    run_smoke( $conf->{continue}, @ARGV );
 }
 
-sub call_mkovz {
+sub genrpt {
     return unless $options{run};
-    local @ARGV = ( 'nomail', $conf->{ddir} );
-    push  @ARGV, $conf->{locale} ? $conf->{locale} : "";
-    push  @ARGV, $conf->{defaultenv} if $conf->{defaultenv};
-    my $mkovz = File::Spec->catfile( $FindBin::Bin, 'mkovz.pl' );
-    local $0 = $mkovz;
-    do $mkovz or die "Error in mkovz.pl: $@";
+    my $reporter = Test::Smoke::Reporter->new( $conf );
+    $reporter->write_to_file;
 }
 
 sub mailrpt {
@@ -207,33 +215,47 @@ sub mailrpt {
     $mailer->mail or warn "[$conf->{mail_type}] " . $mailer->error;
 }
 
+sub archiverpt {
+    return unless $conf->{archive};
+    return unless exists $conf->{adir};
+    return if $conf->{adir} eq "";
+    -d $conf->{adir} or do {
+        mkpath( $conf->{adir}, 0, 0775 ) or 
+            die "Cannot create '$conf->{adir}': $!";
+    };
 
-sub configs_from_log {
-    my( $dir ) = @_;
+    my $patch_level = get_patch( $conf->{ddir} );
+    $patch_level =~ tr/ //sd;
 
-    my $log_name = File::Spec->catfile( $dir, "mktest.out" );
-    my @configs;
-    local *LOG;
-    open LOG, "< $log_name" or die "Cannot continue: $!";
-    my( $smoke, $finish ) = ( 0, undef );
-    while ( <LOG> ) {
-        /^Smoking patch (\d+)/ and $smoke = $1;
-        /^Finished smoking patch $smoke/ and $finish = 1;
-        /^Configuration:\s+(.*)/ or next;
-        push @configs, $1;
+    my $archived_rpt = "rpt${patch_level}.rpt";
+
+    # Do not archive if it is already done
+    return if -f File::Spec->catfile( $conf->{adir}, $archived_rpt );
+
+    copy( File::Spec->catfile( $conf->{ddir}, 'mktest.rpt' ),
+          File::Spec->catfile( $conf->{adir}, $archived_rpt ) ) or
+        die "Cannot copy to '$archived_rpt': $!";
+
+    SKIP_LOG: {
+        my $archived_log = "log${patch_level}.log";
+        last SKIP_LOG unless defined $conf->{lfile};
+        last SKIP_LOG 
+            unless -f $conf->{lfile};
+        copy( $conf->{lfile},
+              File::Spec->catfile( $conf->{adir}, $archived_log ) ) or
+            die "Cannot copy to '$archived_log': $!";
     }
-    close LOG;
-    pop @configs unless $finish;
-    return @configs;
 }
 
 sub snapshot_name {
     my( $plevel ) = $options{snapshot} =~ /(\d+)/;
     my $sfile = $conf->{sfile};
     if ( $sfile ) {
-        $sfile =~ s/\d+/$plevel/;
+        $sfile =~ s/([-@])\d+\./$1$plevel./;
     } else {
-        $sfile = "perl\@$plevel.$conf->{snapext}";
+        my $sep = $conf->{is56x} ? '562-' : '@';
+        my $ext = $conf->{snapext} || 'tar.gz';
+        $sfile = "perl${sep}${plevel}.$ext";
     }
     return $sfile;
 }
@@ -244,7 +266,7 @@ L<README>, L<FAQ>, L<configsmoke.pl>, L<mktest.pl>, L<mkovz.pl>
 
 =head1 REVISION
 
-$Id: smokeperl.pl 313 2003-08-02 01:31:57Z abeltje $
+$Id: smokeperl.pl 662 2004-03-26 08:29:59Z abeltje $
 
 =head1 COPYRIGHT
 
