@@ -2,25 +2,31 @@
 use strict;
 $| = 1;
 
-# $Id: smokestatus.pl 479 2003-10-19 12:28:15Z abeltje $
+# $Id: smokestatus.pl 681 2004-04-26 20:06:59Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.006';
+$VERSION = '0.014';
 
 use Cwd;
-use File::Spec;
+use Time::Local;
+use File::Spec::Functions;
 use File::Path;
 use File::Copy;
 use FindBin;
-use lib File::Spec->catdir( $FindBin::Bin, 'lib' );
+use lib catdir( $FindBin::Bin, 'lib' );
 use lib $FindBin::Bin;
 use Test::Smoke;
-use Test::Smoke::Util qw( get_patch do_pod2usage parse_report_Config );
+use Test::Smoke::Reporter;
+use Test::Smoke::Util qw( 
+    do_pod2usage time_in_hhmm calc_timeout
+    get_patch parse_report_Config );
 
 my $myusage = "Usage: $0 -c [smokeconfig]";
 use Getopt::Long;
+Getopt::Long::Configure( 'bundling' );
 my %opt = (
     dir     => undef,
     config  => undef,
+    matrix  => undef,
     help    => 0,
     man     => 0,
 );
@@ -49,6 +55,10 @@ smokestatus.pl - Check the status of a running smoke
 F<smokestatus.pl> uses the configuration file created by
 F<configsmoke.pl>.
 
+=item * B<Output options>
+
+    -m | --matrix            Add the letter-matrix when possible
+
 =item * B<General options>
 
     -d | --dir <directory>   Specify where the *_config files are
@@ -64,21 +74,22 @@ reports.
 
 =cut
 
-GetOptions( \%opt,
-    'all|a', 'running|r',
-    'dir|d=s',
+GetOptions( \%opt, qw(
+    all|a      running|r
+    dir|d=s
+    matrix|m
 
-    'help|h', 'man',
+    help|h     man
 
-    'config|c:s',
-) or do_pod2usage( verbose => 1, myusage => $myusage );
+    config|c:s
+)) or do_pod2usage( verbose => 1, myusage => $myusage );
 
 do_pod2usage( verbose => 1, exitval => 1, myusage => $myusage )
     unless $opt{all} || $opt{running} || defined $opt{config};
 $opt{ man} and do_pod2usage( verbose => 2, exitval => 0, myusage => $myusage );
 $opt{help} and do_pod2usage( verbose => 1, exitval => 0, myusage => $myusage );
 
-defined $opt{dir} or $opt{dir} = File::Spec->curdir;
+defined $opt{dir} or $opt{dir} = curdir();
 $opt{dir} && -d $opt{dir} or $opt{dir} = '';
 $opt{dir} ||= $FindBin::Bin;
 
@@ -90,8 +101,10 @@ foreach my $config ( @configs ) {
     %opt = %save_opt;
     $opt{config} = $config;
     process_args();
-    print "Checking status for configuration '$opt{config}'\n";
-    my $rpt  = parse_out( $opt{ddir} ) or do {
+    print "\n" unless $config eq $configs[0];
+    my $pver = $opt{perl_version} ? " ($opt{perl_version})" : "";
+    print "Checking status for configuration '$opt{config}'$pver\n";
+    my $rpt  = parse_out( { ddir => $opt{ddir} } ) or do {
         guess_status( $opt{ddir}, $opt{adir}, $opt{config} );
         next;
     };
@@ -102,33 +115,55 @@ foreach my $config ( @configs ) {
     printf "  Change number $rpt->{patch} started on %s.\n", 
            scalar localtime( $rpt->{started} );
 
-    print "    $rpt->{count} out of $ccnt configurations finished",
-          $rpt->{count} ? " in $rpt->{time}.\n" : ".\n";
+    print "    $rpt->{ccount} out of $ccnt configurations finished",
+          $rpt->{ccount} ? " in $rpt->{time}.\n" : ".\n";
 
     printf "    $rpt->{fail} configuration%s showed failures%s.\n",
            ($rpt->{fail} == 1 ? "":"s"), $rpt->{stat} ? " ($rpt->{stat})":""
-        if $rpt->{count};
+        if $rpt->{ccount};
 
     printf "    $rpt->{running} failure%s in the running configuration.\n",
            ($rpt->{running} == 1 ? "" : "s")
         if exists $rpt->{running};
 
-    my $todo = $ccnt - $rpt->{count};
-    my $todo_time = $rpt->{avg} eq 'unknown' ? '.' :
-        ", estimated completion in " . time_in_hhmm( $todo * $rpt->{avg} );
+    my $todo = $ccnt - $rpt->{ccount};
+    my $est_curr = $rpt->{avg} > 0
+        ? $rpt->{avg} - ( $rpt->{rtime} - $rpt->{ccount}*$rpt->{avg} ) : 0;
+    my $est_todo = $todo > 0 && $rpt->{avg} > 0
+        ? ( (($todo - 1) * $rpt->{avg}) + $est_curr ) : 0;
+    $est_todo > $todo * $rpt->{avg} and $est_todo = $todo * $rpt->{avg};
+    my $killtime = calc_timeout( $conf->{killtime}, $rpt->{started} )
+        ? timeout_msg( $conf->{killtime}, $rpt->{started } )
+        : "";
+    
+    my $todo_time = $rpt->{avg} <= 0  ? '.' : 
+        $est_todo <= 0 
+            ? has_lck( $config )
+                ? ", smoke looks hanging delay " . time_in_hhmm( -$est_todo )
+                : ", smoke looks terminated${killtime}."
+            : ", estimated completion in " . time_in_hhmm( $est_todo );
+
     printf "    $todo configuration%s to finish$todo_time\n",
            $todo == 1 ? "" : "s"
         if $todo;
 
     printf "    Average smoke duration: %s.\n", time_in_hhmm( $rpt->{avg} )
-        if $rpt->{count};
+        if $rpt->{ccount};
+
+    if ( $rpt->{ccount} > 0 && $opt{matrix} ) {
+        printf "  Matrix, using %s:\n", $rpt->{reporter}->ccinfo;
+        print join "", map "  $_\n" 
+            => split /\n/, $rpt->{reporter}->smoke_matrix;
+        print join "", map "  $_\n" 
+            => split /\n/, $rpt->{reporter}->bldenv_legend;
+    }
 }
 
 sub guess_status {
     my( $ddir, $adir, $config ) = @_;
     ( my $patch = get_patch( $ddir ) || "" ) =~ s/\?//g;
     if ( $patch && $adir ) {
-        my $a_rpt = File::Spec->catfile( $adir, "rpt${patch}.rpt" );
+        my $a_rpt = catfile( $adir, "rpt${patch}.rpt" );
         my $mtime = -e $a_rpt ? (stat $a_rpt)[9] : undef;
         if ( $mtime ) {
             local *REPORT;
@@ -150,103 +185,38 @@ sub guess_status {
 }
 
 sub parse_out {
-    my( $ddir ) = @_;
-    my $mktest_out = File::Spec->catfile( $ddir, 'mktest.out' );
-    local *MKTESTOUT;
-    open MKTESTOUT, "< $mktest_out" or return;
-    my( %rpt, $cfg, $cnt, $start );
-    while ( <MKTESTOUT> ) {
-        m/^\s*$/ and next;
-        m/^-+$/  and next;
-        s/\s*$//;
+    my( $conf ) = @_;
 
-        next if /^MANIFEST/ || /^PERLIO\s*=/ ||
-                /^Skipped this configuration/;
+    return unless -f catfile $conf->{ddir}, 'mktest.out';
 
-        if  ( my( $patch ) = /^Smoking patch\s* (\d+\S*)/ ) {
-            $rpt{patch} = $patch;
-            next;
-        }
-
-        if ( my( $status, $time ) = /(Started|Stopped) smoke at (\d+)/ ) {
-            if ( $status eq "Started" ) {
-                $start = $time;
-                $rpt{started} ||= $time;
-            } else {
-                $rpt{secs} += ($time - $start) if defined $start;
-            }
-            next;
-        }
-
-        if ( s/^\s*Configuration:\s*// ) {
-            $rpt{config}->{ $cfg } = $cnt if defined $cfg;
-            $cfg = $_; $cnt = 0;
-            next;
-        }
-
-        if ( /^Finished smoking \d+/ ) {
-            $rpt{config}{ $cfg } = $cnt;
-            $rpt{finished} = "Finished";
-            next;
-        }
-
-        if ( my( $status, $mini ) = 
-             m/^\s*Unable\ to
-               \ (?=([cbmt]))(?:build|configure|make|test)
-               \ (anything\ but\ mini)?perl/x) {
-            $mini and $status = uc $status; # M for no perl but miniperl
-            $cnt = $status;
-            next;
-        }
-        $cnt = 0, next if /^\s*All tests successful/;
-        $cnt++,   next if /FAILED|DIED/;
-	next;
-    }
-    close MKTESTOUT;
+    my $reporter = Test::Smoke::Reporter->new( $conf );
+    my %rpt = %{ $reporter->{_rpt} };
 
     $rpt{finished} ||= "Busy";
-    $rpt{count} = scalar keys %{ $rpt{config} };
-    $rpt{avg}   = $rpt{count} ? $rpt{secs} / $rpt{count} : 'unknown';
+    $rpt{ccount} = scalar keys %{ $rpt{statcfg} };
+    $rpt{avg}   = $rpt{ccount} ? $rpt{secs} / $rpt{ccount} : 0;
     $rpt{time}  = time_in_hhmm( $rpt{secs} );
+    $rpt{rtime} = time() - $rpt{started};
     $rpt{fail} = 0; $rpt{stat} = { };
-    foreach my $config ( keys %{ $rpt{config} } ) {
 
-        if ( $rpt{config}{ $config } ) {
-            $rpt{config}{ $config } = "F" 
-                if $rpt{config}{ $config } =~ /^\d+$/;
+    my $fcnt = 0;
+    foreach my $config ( keys %{ $rpt{statcfg} } ) {
+
+        if ( $rpt{statcfg}{ $config } ) {
+            $fcnt = $rpt{statcfg}{ $config };
+            $rpt{statcfg}{ $config } = "F" 
+                if $rpt{statcfg}{ $config } =~ /^\d+$/;
 
             $rpt{fail}++;
-            $rpt{stat}->{ $rpt{config}{ $config } }++;
+            $rpt{stat}->{ $rpt{statcfg}{ $config } }++;
         }
     }
     $rpt{stat} = join "", sort keys %{ $rpt{stat} };
 
-    $rpt{running} = $cnt unless exists $rpt{config}->{ $cfg };
-
+    $rpt{reporter} = $reporter;
     return \%rpt    
 }
 
-sub time_in_hhmm {
-    my $diff = shift;
-
-    my $digits = $diff =~ /\./ ? 3 : 0;
-    my $days = int( $diff / (24*60*60) );
-    $diff -= 24*60*60 * $days;
-    my $hour = int( $diff / (60*60) );
-    $diff -= 60*60 * $hour;
-    my $mins = int( $diff / 60 );
-    $diff -=  60 * $mins;
-
-    my @parts;
-    $days and push @parts, sprintf "%d day%s",   $days, $days == 1 ? "" : 's';
-    $hour and push @parts, sprintf "%d hour%s",  $hour, $hour == 1 ? "" : 's';
-    $mins and push @parts, sprintf "%d minute%s",$mins, $mins == 1 ? "" : 's';
-    $diff && !$days && !$hour and
-        push @parts, sprintf "%.${digits}f seconds", $diff;
-
-    return join " ", @parts;
-}
-
 sub get_configs {
     local *DH;
     opendir DH, $opt{dir} or return;
@@ -263,6 +233,11 @@ sub get_lcks {
     return sort @list;
 }
 
+sub has_lck {
+    ( my $lck = shift ) =~ s/_config\z/.lck/;
+    return -f File::Spec->catfile( $opt{dir}, $lck );
+}
+
 sub process_args {
     return unless defined $opt{config};
 
@@ -284,6 +259,32 @@ sub process_args {
     }
 }
 
+sub timeout_msg {
+    my( $killtime, $from ) = @_;
+
+    defined $from or $from = time;
+    if ( $killtime =~ /^\+(\d+):(\d+)/ ) {
+        my( $hh, $mm ) = ( $1, $2 );
+        $from += 60 * $mm;
+        $from += 60 * 60 * $hh;
+        return " from " . localtime $from;
+    } else {
+        my @lt = localtime $from;
+        my( $hh, $mm ) = $killtime =~ /(\d+):(\d+)/;
+        my $time_min = 60 * $hh + $mm;
+        my( $now_m, $now_h ) = @lt[1, 2];
+        my $now_min = 60 * $now_h + $now_m;
+        my $kill_min = $time_min - $now_min;
+        $kill_min += 60 * 24 if $kill_min < 0;
+
+        $hh = int( $kill_min / 60 );
+        $mm = $kill_min % 60;
+        @lt[ 1, 2] = ( $mm, $hh );
+
+        return " at " . localtime timelocal @lt;
+    }
+}
+
 =head1 COPYRIGHT
 
 (c) 2002-2003, All rights reserved.
